# include "CaDA_Motor.h"

Motor::Motor()
{
}

Motor::Motor(uint8_t _port)
{
    switch (_port)
    {
        case portA:
            _ina = pinH1A;
			_inb = pinH1B;
            break;
        case portB:
            _ina = pinH2A;
			_inb = pinH2B;
            break;
        default:
            break;
    }
}

//电机自定义IO实例化
Motor::Motor(uint8_t _INA ,uint8_t _INB)
{
    _ina = _INA;
    _inb = _INB;
}


//初始化
void Motor::begin()
{
	pinMode(_ina, OUTPUT);
    pinMode(_inb, OUTPUT);
    analogWriteFreq(20000); //PMW最大40K驱动频率
}

//电机端口初始化
void Motor::setMotorPort(uint8_t _port)
{
    switch (_port)
    {
        case portA:
            _ina = pinH1A;
			_inb = pinH1B;
            break;
        case portB:
            _ina = pinH2A;
			_inb = pinH2B;
            break;
        default:
            break;
    }
	pinMode(_ina, OUTPUT);
    pinMode(_inb, OUTPUT);
    analogWriteFreq(20000); //PMW最大40K驱动频率
}

//设置电机方向，1为正转，2为反转
void Motor::setMotorDir(uint8_t dir)
{
    _dir = dir;
}

//单电机速度设置
void Motor::setMotorSpeed(uint8_t speed)
{

    _speed = ~speed;  //输入速度取反

}

void Motor::setMotorMode(uint8_t cmd)  //电机运行函数
{
    switch (cmd)
    {
        case FORWARD:
            _cw();
            break;
        case BACKWARD:
            _ccw();
            break;
        case BRAKE:
            _brake();
            break;
        case RELEASE:
            _release();
            break;
        default:
            break;
    }
}


//电机正转运行
void Motor::_cw()
{
    if(_dir == FORWARD)
    {
        digitalWrite(_ina, HIGH);
        analogWrite(_inb, _speed);
    }
    else if(_dir == BACKWARD)
    {
        analogWrite(_ina, _speed);
        digitalWrite(_inb, HIGH);
    }
}

//电机正转运行
void Motor::_cw(uint8_t speed)
{
    if (_dir == FORWARD)
    {
        digitalWrite(_ina, HIGH);
        analogWrite(_inb, speed);
    }
    else if (_dir == BACKWARD)
    {
        analogWrite(_ina, speed);
        digitalWrite(_inb, HIGH);
    }
}
//电机反转运行
void Motor::_ccw()
{
    if(_dir == FORWARD)
    {
        analogWrite(_ina, _speed);
        digitalWrite(_inb, HIGH);
    }
    else if(_dir == BACKWARD)
    {
        

        digitalWrite(_ina, HIGH);
        analogWrite(_inb, _speed);
    }
    
}
//电机反转运行
void Motor::_ccw(uint8_t speed)
{
    if(_dir == FORWARD)
    {
        analogWrite(_ina, speed);
        digitalWrite(_inb, HIGH);
    }
    else if(_dir == BACKWARD)
    {
        digitalWrite(_ina, HIGH);
        analogWrite (_inb, speed);
    }
   
}
//电机刹车
void Motor::_brake()
{
    digitalWrite(_ina, HIGH);
    digitalWrite(_inb, HIGH);
}
//电机释放
void Motor::_release()
{
    digitalWrite(_ina, LOW);
    digitalWrite(_inb, LOW);
}
